#include <sys/stat.h>
#include <unistd.h>

#include "wave.h"
#include "defaults.h"
#include "gsm/inc/gsm.h"

char *USAGE =

"USAGE %s [options] <GSM-file>\n"
"Converts a GSM file to a 16 bit mono wave-file.\n"
"Options: -s<sample rate> in Hz, kHz or f (11025 Hz) (default 8000)\n"
"         -o<output file>, -h print this message, -v verbose mode\n";

struct stat st;

gsm handle;
gsm_frame buf;
gsm_signal sample[160];

int main(int argc,char **argv)
{
  wavfile w,g;
  LONGINT i,j,n,s;
  char c;
  VAL x;

  setexitmsg(argv[0]);
  w.f=stdout;
  w.rate=8000;
  w.bits=16;
  w.bytes=2;
  
  while(1)
  {
    c=getopt(argc,argv,"s:o:hv");
    if(stdopt(c,USAGE)) break;
    switch(c)
    {
      case 's':
        x=getfreq(optarg);
        if(x<0)
        {
          fprintf(stderr,"%s: Invalid sampling rate\n",PRG);
          errexit(1);
        };
        w.rate=rint(x);
        break;
      case 'o':
        w.f=fileopen(optarg,"wb",0);
        break;
    };
  };
  
  if(optind<argc-1 || optind>=argc) errexit(ERR_ARG);

  if(stat(argv[optind],&st)) errexit(ERR_READ);
  s=st.st_size;
  if(s % 33) {
    fprintf(stderr,"%s: Input file is no GSM file.\n",PRG);
    errexit(ERR_FORMAT);
  }
  g.f=fileopen(argv[optind],"r",".gsm");
  w.n=160*(s/33);
  writeheader(&w);

  if (!(handle = gsm_create())) exit(1);

  for(i=0;i<s/33;i++) {
    if(fread(buf,sizeof(buf),1,g.f)!=1 ||
       gsm_decode(handle, buf, sample) < 0) {
      fprintf(stderr,"%s: Error decoding input file.\n",PRG);
      errexit(ERR_FORMAT);
    }
    for(j=0;j<160;j++) {
      wavbinput(&w,sample[j]);
    };
  }; 

  errexit(0);
}
    

